USE AdventureWorks2012;
GO

/*
	Rilevare la frammentazione
	avg_fragmentation_in_percent: ideale valore basso
	fragment_count: ideale valore basso
	avg_fragment_size_in_pages: ideale valore alto
*/

SELECT * FROM sys.dm_db_index_physical_stats
    (DB_ID(N'AdventureWorks2012'), OBJECT_ID(N'Person.Address'), NULL, NULL , 'DETAILED');

SELECT * FROM sys.dm_db_index_physical_stats
    (DB_ID(N'AdventureWorks2012'), NULL, NULL, NULL , 'DETAILED');


/*
	Ricostruisce o riorganizza selettivamente gli indici di un database
	ATTENZIONE: aggiungere lo USE DATABASE in testa
	Fonte: BOL - sys.dm_db_index_physical_stats
*/

SET NOCOUNT ON;
DECLARE @objectid int;
DECLARE @indexid int;
DECLARE @partitioncount bigint;
DECLARE @schemaname nvarchar(130); 
DECLARE @objectname nvarchar(130); 
DECLARE @indexname nvarchar(130); 
DECLARE @partitionnum bigint;
DECLARE @partitions bigint;
DECLARE @frag float;
DECLARE @command nvarchar(4000); 
DECLARE @dbid smallint

SET @dbid = DB_ID()

SELECT object_id AS objectid, index_id AS indexid, 
	partition_number AS partitionnum, avg_fragmentation_in_percent AS frag
INTO #work_to_do
FROM sys.dm_db_index_physical_stats (@dbid, NULL, NULL , NULL, 'DETAILED')
WHERE avg_fragmentation_in_percent > 10.0 AND index_id > 0;

DECLARE partitions CURSOR FOR 
	SELECT * 
	FROM #work_to_do;

OPEN partitions;

WHILE (1=1)
    BEGIN;
        FETCH NEXT FROM partitions INTO @objectid, @indexid, @partitionnum, @frag;

        IF @@FETCH_STATUS < 0 BREAK;

        SELECT @objectname = QUOTENAME(o.name), @schemaname = QUOTENAME(s.name)
        FROM sys.objects AS o
        JOIN sys.schemas as s ON s.schema_id = o.schema_id
        WHERE o.object_id = @objectid;

        SELECT @indexname = QUOTENAME(name)
        FROM sys.indexes
        WHERE  object_id = @objectid AND index_id = @indexid;

        SELECT @partitioncount = count (*)
        FROM sys.partitions
        WHERE object_id = @objectid AND index_id = @indexid;

        IF @frag < 30.0
            SET @command = N'ALTER INDEX ' + @indexname + N' ON ' + @schemaname + N'.' + @objectname + N' REORGANIZE';
        IF @frag >= 30.0
            SET @command = N'ALTER INDEX ' + @indexname + N' ON ' + @schemaname + N'.' + @objectname + N' REBUILD';

        IF @partitioncount > 1
            SET @command = @command + N' PARTITION=' + CAST(@partitionnum AS nvarchar(10));

        EXEC (@command);

        PRINT N'Executed: ' + @command;
    END;

CLOSE partitions;
DEALLOCATE partitions;

DROP TABLE #work_to_do;
GO
